@@ -1,14 +1,14 @@
 package List::MoreUtils;
 
-use 5.00503;
+use 5.008003;
 use strict;
 use Exporter   ();
 use DynaLoader ();
 
-use vars qw{ $VERSION $XS_VERSION @ISA @EXPORT_OK %EXPORT_TAGS };
+use vars qw{ $VERSION @ISA @EXPORT_OK %EXPORT_TAGS };
 BEGIN {
-    $VERSION   = '0.33_007';
-    $XS_VERSION   = eval $VERSION;
+    $VERSION   = '0.400';
+    # $VERSION   = eval $VERSION;
     @ISA       = qw{ Exporter DynaLoader };
     @EXPORT_OK = qw{
         any all none notall true false
@@ -21,19 +21,13 @@ BEGIN {
         pairwise natatime
         mesh zip uniq distinct
         minmax part
+        sort_by nsort_by
+        bsearch
     };
     %EXPORT_TAGS = (
         all => \@EXPORT_OK,
     );
 
-    # Mention $a and $b within the caller's package to eliminate the
-    # "used only once" warnings. RT 76749
-    {
-        no strict 'refs';
-        ${caller().'::a'} = ${caller().'::a'};
-        ${caller().'::b'} = ${caller().'::b'};
-    }
-
     # Load the XS at compile-time so that redefinition warnings will be
     # thrown correctly if the XS versions of part or indexes loaded
     eval {
@@ -239,8 +233,8 @@ sub pairwise (&\@\@) {
     # Loop iteration limit
     my $limit = $#A > $#B? $#A : $#B;
 
-    local( *$caller_a, *$caller_b );
     # This map expression is also the return value
+    local( *$caller_a, *$caller_b );
     map {
         # Assign to $a, $b as refs to caller's array elements
         ( *$caller_a, *$caller_b ) = \( $A[$_], $B[$_] );
@@ -348,6 +342,34 @@ sub part (&@) {
     return @parts;
 }
 
+sub bsearch(&@) {
+    my $code = shift;
+
+    my $rc;
+    my $i = 0;
+    my $j = @_;
+    do {
+        my $k = int(($i + $j) / 2);
+
+        $k >= @_ and return;
+
+        local *_ = \$_[$k];
+        $rc = $code->();
+
+        $rc == 0 and
+            return wantarray ? $_ : 1;
+
+        if ($rc < 0) {
+            $i = $k + 1;
+        }
+	else {
+            $j = $k - 1;
+        }
+    } until $i > $j;
+
+    return;
+}
+
 sub _XScompiled {
     return 0;
 }
@@ -363,6 +385,22 @@ die $@ if $@;
 *zip         = \&mesh;
 *distinct    = \&uniq;
 
+sub sort_by(&@) {
+    my ($code, @list) = @_;
+    return map { $_->[0] }
+          sort { $a->[1] cmp $b->[1] }
+           map { [$_, scalar($code->())] }
+               @list;
+}
+
+sub nsort_by(&@) {
+    my ($code, @list) = @_;
+    return map { $_->[0] }
+          sort { $a->[1] <=> $b->[1] }
+           map { [$_, scalar($code->())] }
+               @list;
+}
+
 1;
 
 __END__
@@ -410,6 +448,8 @@ BLOCK. Sets C<$_> for each item in LIST in turn:
 
 Returns false otherwise, or if LIST is empty.
 
+B<The behaviour without LIST needs to be discussed!>
+
 =item all BLOCK LIST
 
 Returns a true value if all items in LIST meet the criterion given through
@@ -420,6 +460,8 @@ BLOCK, or if LIST is empty. Sets C<$_> for each item in LIST in turn:
 
 Returns false otherwise.
 
+B<The behaviour without LIST needs to be discussed!>
+
 =item none BLOCK LIST
 
 Logically the negation of C<any>. Returns a true value if no item in LIST meets
@@ -442,6 +484,8 @@ turn:
 
 Returns false otherwise, or if LIST is empty.
 
+B<The behaviour without LIST needs to be discussed!>
+
 =item true BLOCK LIST
 
 Counts the number of elements in LIST for which the criterion in BLOCK is true.
@@ -564,7 +608,7 @@ Returns the first element in LIST for which BLOCK evaluates to true. Each
 element of LIST is set to C<$_> in turn. Returns C<undef> if no such element
 has been found.
 
-C<first_value> is an alias for C<firstval>.
+C<first_val> is an alias for C<firstval>.
 
 =item lastval BLOCK LIST
 
@@ -574,7 +618,7 @@ Returns the last value in LIST for which BLOCK evaluates to true. Each element
 of LIST is set to C<$_> in turn. Returns C<undef> if no such element has been
 found.
 
-C<last_value> is an alias for C<lastval>.
+C<last_val> is an alias for C<lastval>.
 
 =item pairwise BLOCK ARRAY1 ARRAY2
 
@@ -710,6 +754,44 @@ Negative values are only ok when they refer to a partition previously created:
     my $i    = 0;
     my @part = part { $idx[$++ % 3] } 1 .. 8; # [1, 4, 7], [2, 3, 5, 6, 8]
 
+=item bsearch BLOCK LIST
+
+Performs a binary search on LIST which must be a sorted list of values. BLOCK
+must return a negative value if the current element (stored in C<$_>) is smaller,
+a positive value if it is bigger and zero if it matches.
+
+Returns a boolean value in scalar context. In list context, it returns the element
+if it was found, otherwise the empty list.
+
+=item sort_by BLOCK LIST
+
+Returns the list of values sorted according to the string values returned by the
+KEYFUNC block or function. A typical use of this may be to sort objects according
+to the string value of some accessor, such as
+
+    sort_by { $_->name } @people
+
+The key function is called in scalar context, being passed each value in turn as
+both $_ and the only argument in the parameters, @_. The values are then sorted
+according to string comparisons on the values returned.
+This is equivalent to
+
+    sort { $a->name cmp $b->name } @people
+
+except that it guarantees the name accessor will be executed only once per value.
+One interesting use-case is to sort strings which may have numbers embedded in them
+"naturally", rather than lexically.
+
+    sort_by { s/(\d+)/sprintf "%09d", $1/eg; $_ } @strings
+
+This sorts strings by generating sort keys which zero-pad the embedded numbers to
+some level (9 digits in this case), helping to ensure the lexical sort puts them
+in the correct order.
+
+=item nsort_by BLOCK LIST
+
+Similar to sort_by but compares its key values numerically.
+
 =back
 
 =head1 EXPORTS
@@ -761,7 +843,68 @@ pure-Perl or possibly both).
 
 Bugs should always be submitted via the CPAN bug tracker.
 
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=List-MoreUtils>
+You can find documentation for this module with the perldoc command.
+
+    perldoc List::MoreUtils
+
+You can also look for information at:
+
+=over 4
+
+=item * RT: CPAN's request tracker
+
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=List-MoreUtils>
+
+=item * AnnoCPAN: Annotated CPAN documentation
+
+L<http://annocpan.org/dist/List-MoreUtils>
+
+=item * CPAN Ratings
+
+L<http://cpanratings.perl.org/l/List-MoreUtils>
+
+=item * CPAN Search
+
+L<http://search.cpan.org/dist/List-MoreUtils/>
+
+=item * Got Repository
+
+L<https://github.com/perl5-utils/List-MoreUtils>
+
+=back
+
+=head2 Where can I go for help?
+
+If you have a bug report, a patch or a suggestion, please open a new
+report ticket at CPAN (but please check previous reports first in case
+your issue has already been addressed).
+
+Report tickets should contain a detailed description of the bug or
+enhancement request and at least an easily verifiable way of
+reproducing the issue or fix. Patches are always welcome, too - and
+it's cheap to send pull-requests on GitHub. Please keep in mind that
+code changes are more likely accepted when they're bundled with an
+approving test.
+
+If you think you've found a bug then please read
+"How to Report Bugs Effectively" by Simon Tatham:
+L<http://www.chiark.greenend.org.uk/~sgtatham/bugs.html>.
+
+=head2 Where can I go for help with a concrete version?
+
+Bugs and feature requests are accepted against the latest version
+only. To get patches for earlier versions, you need to get an
+agreement with a developer of your choice - who may or not report the
+issue and a suggested fix upstream (depends on the license you have
+chosen).
+
+=head2 Business support and maintenance
+
+For business support you can contact Jens via his CPAN email
+address rehsackATcpan.org. Please keep in mind that business
+support is neither available for free nor are you eligible to
+receive any support based on the license distributed with this
+package.
 
 =head1 THANKS
 
@@ -794,9 +937,6 @@ Perl-implementation.
 Robin Huston kindly fixed a bug in perl's MULTICALL API to make the
 XS-implementation of part() work.
 
-Reini Urban (rurban) and David J Oswald (davido) applied various outstanding
-patches and fixed various tickets. The RT queue still has 57 open issues.
-
 =head1 TODO
 
 A pile of requests from other people is still pending further processing in
@@ -843,6 +983,8 @@ L<List::Util>
 
 =head1 AUTHOR
 
+Jens Rehsack E<lt>rehsack AT cpan.orgE<gt>
+
 Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
 Tassilo von Parseval E<lt>tassilo.von.parseval@rwth-aachen.deE<gt>
@@ -852,6 +994,7 @@ Tassilo von Parseval E<lt>tassilo.von.parseval@rwth-aachen.deE<gt>
 Some parts copyright 2011 Aaron Crane.
 
 Copyright 2004 - 2010 by Tassilo von Parseval
+Copyright 2013 by Jens Rehsack
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.4 or,
